/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import com.funambol.mail.MessageFlags;
import com.funambol.mailclient.config.*;
import com.funambol.mailclient.sm.SyncClientConfig;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.AlarmManager;
import com.funambol.mailclient.ui.controller.Killable;
import com.funambol.mailclient.ui.controller.MessageMediaManager;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.controller.MessageHandler;
import com.funambol.mailclient.ui.controller.LifeCycleController;
import com.funambol.mailclient.ui.view.FunSplashScreen;
import com.funambol.mailclient.loc.*;
import com.funambol.mailclient.ui.view.ExitPopupAction;
import com.funambol.mailclient.ui.view.FunModalPopup;
import com.funambol.util.Log;
import com.funambol.util.RMSAppender;
import com.funambol.util.SocketAppender;
import com.funambol.mail.Store;

import com.funambol.push.CTPService;
import com.funambol.push.OTAService;
import com.funambol.push.PushConfig;
import java.io.IOException;
import java.io.InputStream;

import java.util.Date;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.PushRegistry;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

public class Funambol extends MIDlet implements Killable, Runnable {
    
    private MailClientConfig mailClientConfig;
    private OTAService otaService;
    private CTPService ctpService;
    //private OTAService OTAConfigListener;
    private MessageHandler messageHandler = null;
    
    private FunSplashScreen splashScreen;
    
    
    public Funambol() {
        //startAppProbe = System.currentTimeMillis();
        splashScreen = new FunSplashScreen();
        Display.getDisplay(this).setCurrent(splashScreen);
    }
    
    protected void startApp() {
        if (LifeCycleController.isPaused == false) {
            // init localization for specific language
            //Localization.setMessages(new IT_LocalizedMessages());
            new Thread(this).start();
        } else {
            // Resuming the MIDlet from a PAUSE state
            LifeCycleController.isPaused = false;
            LifeCycleController.getInstance().resume();
        }
    }
    
    public void setSplashPhase(String message, int iconNumber) {
        splashScreen.setInitPhase(message, iconNumber);
    }
    
    public void setSplashPhase(String[] message, int iconNumber) {
        
        splashScreen.setInitPhase(message, iconNumber);
    }
    
    
    private void startSync(final boolean ok2Sync) {
        if (ok2Sync) {
            if (!UIController.otaStarted) {
                if (mailClientConfig.isSyncOnStartupEnabled()) {
                    Log.info("starting sync");
                    UIController.sync(true);
                } else {
                    Log.info("sync on startup disabled");
                }
            } else {
                Log.info("[FunambolMIDlet] Ota started the application... sync using SAN");
                
                // Workaround to avoid startsync failing with fake SMS incoming:
                // i.e. Sprint MotoV3m
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                }
                if ((!SyncClient.getSyncClient().isBusy()) && (!otaService.smsHandling)) {
                    Log.info("Sprint fix - starting sync due to fake OTA start");
                    UIController.sync(true);
                }
                
            }
        }
    }
    
    public void destroyApp(boolean unconditional) {
        Log.debug("DestroyAPP");
        FunModalPopup popup =
                new FunModalPopup(Localization.getMessages().POPUP_EXIT_TITLE,
                Localization.getMessages().POPUP_EXIT_TEXT,
                new ExitPopupAction());
        UIController.showModalPopup(popup, UIController.getInboxMessageList());
        
    }
    
    /**
     * Pausing the app means to put the Controller in paused mode.
     * In this way it will perform all the operations needed later when resuming
     */
    protected void pauseApp() {
        
        LifeCycleController.getInstance().pause();
        
    }
    
    /**
     * Exit the application. The exit method is invoked by the Controller
     * to exit the MIDlet
     */
    public void exit() throws MIDletStateChangeException {
        
        otaService.stopService();
        //OTAConfigListener.close();
        String ctp = getAppProperty("CTP");
        
        // getAppProperty can return null if property is not defined,
        // so we need to check if it's not null befor doing the comparison
        if (ctp!=null && ctp.equals("enabled")) {
            ctpService.stopService();
        }
        
        notifyDestroyed();
    }
    
    public void run() {
        
        
        boolean ok2Sync = false;
        
        UIController.midlet=this;
        UIController.display = Display.getDisplay(this);
        //UIController.startAppProbe = startAppProbe;
        UIController.startupFreeMem = Runtime.getRuntime().freeMemory();
        
        // Create the message handler which is responsible for handling push
        // notifications
        messageHandler = new MessageHandler();
        // This code is for backward compatibility. Migration from v6 to v6.5
        // Can be removed in future relase
        ConfigManager.upgrade();
        
        InitConfigManager cm = new InitConfigManager(this);
        cm.initConfig();
        
        UIController.initMessageManager();
        
        // This code is for backward compatibility. Migration from v6 to v6.5
        // Can be removed in future relase
        boolean upgraded = handleUpgrade();
        
        
        // loading opened and closed image, they'll be probably needed later and
        // this minimize the empty inbox for devices like nokia 6630
        Theme.getImageFromFlags(MessageFlags.OPENED );
        Theme.getImageFromFlags(0 );
        
        
        ok2Sync = UIController.initInbox();
        
        cm.initListener();
        
        // Do not resync if we performed an upgrade
        startSync(ok2Sync && !upgraded);
        
        // Start the CTP service (if enabled)
        String ctp = getAppProperty("CTP");
        if (ctp!=null && ctp.equals("enabled")) {
            startCTP();
        }
        
        checkSoundCapabilities();
    }
    
    private void startCTP() {
        
        // If this is the first run, then we better wait for two reasons:
        // 1) the CTP does not authenticate us until a sync authentication has
        //    been performed
        // 2) the device id is generated during the first sync. The device id is
        //    needed to create a proper push configuration
        if (ConfigManager.getFirstRun()) {
            // This is very heuristic...
            // Wait until the sync is done, or long enough for the sync
            // authentication to be finished
            int numSleeps = 0;
            do {
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException ie) {
                }
                numSleeps++;
            } while (SyncClient.getSyncClient().isBusy() && numSleeps < 40);
        }
        
        PushConfig pushConfig = new PushConfig();
        try {
            ConfigManager.load(ConfigManager.PUSH_CONFIG, pushConfig);
        } catch (Exception e) {
            // The configuration does not exist or it is invalid, we restart
            // from an empty one
        } finally {
            // Propagate some info from the other config to the PushConfig
            SyncClientConfig sc = new SyncClientConfig();
            try {
                ConfigManager.load(ConfigManager.SYNCML_CONFIG, sc);
                Account account = mailClientConfig.getMailAccount();
                pushConfig.setCtpUsername(account.getUser());
                pushConfig.setCtpPassword(account.getPassword());
                
                String server = getAppProperty("CTPServer");
                if (server == null) {
                    // Guess the CTP server name from the sync server name
                    server = account.getUrl();
                    // Strip the port and the protocol if present and the resource
                    if (server.startsWith("http://")) {
                        server = server.substring(7, server.length() - 1);
                    }
                    // If we have port number in the address we strip everything
                    // after the port number
                    int pos = server.indexOf(':');
                    if (pos >= 0) {
                        server = server.substring(0, pos);
                    } else {
                        // If we have resource location in the address then we strip
                        // everything after the '/'
                        pos = server.indexOf('/');
                        if (pos >= 0) {
                            server = server.substring(0, pos);
                        }
                    }
                    server = "ctp-" + server;
                } else {
                    int pos = server.indexOf(':');
                    if (pos >= 0) {
                        String portStr = server.substring(pos+1);
                        try {
                            int port = Integer.parseInt(portStr);
                            Log.debug("Setting CTP port to: " + port);
                            pushConfig.setCtpPort(port);
                        } catch (NumberFormatException ne) {
                            Log.error("Bad ctp port parameter: " + portStr);
                        }
                        server = server.substring(0, pos);
                    }
                }
                
                pushConfig.setCtpServer(server);
                pushConfig.setDeviceId(sc.getDeviceId());
                
                ctpService = new CTPService(UIController.getThreadPool(),
                        messageHandler,pushConfig);
                ctpService.startService();
                
                ConfigManager.save(ConfigManager.PUSH_CONFIG, pushConfig);
            } catch (Exception e) {
                e.printStackTrace();
                Log.error("No valid config found. CTPService cannot start");
            }
        }
    }
    
    /**
     * This method checks if an upgrade is needed. The current implementation
     * checks if the Store version is less than 102 and in this case it starts a
     * reset inbox to resynchronize the inbox.
     * The synchronization is executed in a different thread, therefore the
     * method may report a success even if the synchronization fails. On failure
     * the inbox could be partially filled (or even empty), but data would be
     * consistent between client and server.
     *
     * @return true on successful upgrade
     */
    private boolean handleUpgrade() {
        
        // Check if we need to finish upgrade
        if (mailClientConfig.getStoreVersion() < Store.VERSION_102) {
            // The store is still in the old format. We must perform a
            // reset inbox for the upgrade to be completed.
            String msg = Localization.getMessages().UPGRADE_REQUIRED;
            UIController.showModalAlert(msg);
            
            // Reset inbox
            try {
                // This is a non blocking operation, therefore the method exits
                // immediately. If the update is not performed properly (for
                // example because of no network connectivity) then the user is
                // left without any message.
                UIController.resetAccount();
                mailClientConfig.setStoreVersion(Store.LATEST_VERSION);
                ConfigManager.saveConfig();
                return true;
            } catch (Exception e) {
                Log.error("Error while resetting account: " + e.toString());
                return false;
            }
        } else {
            // No update is necessary
            return false;
        }
    }
    
    
    /**
     * @return the splashscreen.
     * this method is used by the threadpoolmonitor to ensure robustness
     * when catching exceptions thrown at startup
     */
    public Displayable getSplashScreen() {
        return splashScreen;
    }
    
    private void checkSoundCapabilities() {
        // Testing ability to play sound
        InputStream is = getClass().getResourceAsStream("/res/newmessage.midi");
        Player player =null;
        try {
            player = Manager.createPlayer(is, "audio/midi");
            UIController.ableToSound = true;
        } catch (SecurityException se) {
            Log.error("[MessageMediaManager] The device carrier restricts " +
                    "Multi Media API usage to trusted applications only: " + se);
        } catch (IOException ex) {
            Log.error(this, "IOException in playSound: " + ex);
            ex.printStackTrace();
        } catch (MediaException ex) {
            Log.error(this, "MediaException in playSound: " + ex);
            ex.printStackTrace();
        } catch (IllegalArgumentException iae) {
            Log.error("[MessageMediaManager] Exception " +
                    "loading sound stream: " + iae);
            iae.printStackTrace();
        } finally {
            is = null;
            player = null;
        }
    }
    
    
    private class InitConfigManager {
        Funambol midlet;
        
        public InitConfigManager(Funambol midlet) {
            this.midlet = midlet;
        }
        
        private void initConfig() {
            UIController.initLogger();
            
            // Initialize the ThreadPoolMonitor
            UIController.initThreadPool();
            
            //long startInitConfig = System.currentTimeMillis();
            // Mail Configuration initInbox
            // We check in Storage then in JAD and at the end load a default config
            try {
                mailClientConfig = ConfigManager.getConfig(midlet);
            } catch (Exception e) {
                Log.error("Invalid mail client configuration: " +
                        "creating a default configuration");
                mailClientConfig = new MailClientConfig();
                e.printStackTrace();
            } finally {
                UIController.mailClientConfig = mailClientConfig;
                if (mailClientConfig.isSchedulerEnabled()) {
                    setAlarm();
                }
                Log.setLogLevel(mailClientConfig.getLogLevel());
                
            }
            
        }
        
        
        private void setAlarm() {
            if (mailClientConfig.getPollInterval()>=1440) {
                int days = (mailClientConfig.getPollInterval()/60)/24;
                AlarmManager.getInstance().startSyncTimerTask(
                        new Date(mailClientConfig.getNextTimeAlarm()), days);
            } else {
                AlarmManager.getInstance().startSyncTimerTask(
                        mailClientConfig.getPollInterval());
            }
        }
        
        
        public void initListener() {
            
            /** Connections detected at start up. */
            try {
                //#ifdef PushRuntime
//#                  //Checking Connection for Push registry listener
//#                  String connection = "sms://:" + getAppProperty("SAN-Port");
//#                  String midletName = UIController.midlet.getClass().getName();
//#
//#                  Log.error("Connection: " + connection);
//#                  Log.error("MIDlet Name: " + midletName);
//#
//#                  try {
//#                      PushRegistry.registerConnection(connection, midletName, "*");
//#                  } catch (IllegalArgumentException iae) {
//#                      Log.error("connection string or filter string " +
//#                                                            "not valid: " + iae);
//#                      iae.printStackTrace();
//#                  } catch (ConnectionNotFoundException cfe) {
//#                      Log.error("Runtime system does not support push delivery "+
//#                                "for the requested connection protocol: " + cfe);
//#                      cfe.printStackTrace();
//#                  } catch (IOException ioe) {
//#                      //If a connection has already been registered, ignores last
//#                      //statement
//#                  } catch (ClassNotFoundException cnf) {
//#                      Log.error("Runtime system does not support push delivery "+
//#                                 "for the requested connection protocol: " +cnf);
//#                      cnf.printStackTrace();
//#                  } catch (SecurityException se) {
//#                      Log.error("User didn't grant registration for push " +
//#                                                              "listener: " + se);
//#                      se.printStackTrace();
//#                  }
//#
//#
                //#endif
                
                // List all the registered connections
                String[] connections = PushRegistry.listConnections(false);
                otaService =
                        new OTAService(getAppProperty("SAN-Port"),
                        UIController.getThreadPool());
                
                otaService.setPushNotificationListener(messageHandler);
                //OTAConfigListener =
                //new OTAMessagesListener(getAppProperty("OTAC-Port"));
                
                if (connections == null || connections.length == 0) {
                    Log.info("[FunambolMIDlet] Application not registered for incoming SMS");
                } else {
                    Log.info(connections.length  + " connections registered.");
                    // list only the active registered connections
                    connections = PushRegistry.listConnections(true);
                    
                    //OTAConfigListener.init();
                    otaService.startService();
                    if (connections == null || connections.length == 0) {
                        Log.info("No active connections.");
                        Log.info("[FunambolMIDlet] User started the application.");
                    } else {
                        Log.info(connections.length  + " active connections.");
                        UIController.otaStarted = true;
                        Log.info("[FunambolMIDlet] Registry connection found: "
                                + connections[connections.length - 1]);
                        if (connections[connections.length - 1]
                                .endsWith(getAppProperty("SAN-Port"))) {
                            OTAService.startViaOTA = true;
                            Log.info("[FunambolMIDlet] SAN Message started the application.");
                        } else {
                            Log.info("[FunambolMIDlet] OTA Config Message started the application.");
                        }
                        
                    }
                }
            } catch (Exception exp) {
                Log.error("Exception occurred listening for SMS: "+exp.getMessage());
                UIController.showErrorAlert(
                        Localization.getMessages().ALERT_SECURITY_EXCEPTION_MESSAGE);
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException ex) {
                    Log.error(this, "interruptedException in Exception in " +
                            "initlistener() method ");
                    ex.printStackTrace();
                }
            }
        }
        
    }
    
}

