/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.controller;

import com.funambol.util.Log;
import java.io.IOException;
import java.io.InputStream;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.midlet.MIDlet;


/**
 * Class in charge of playing a sound for message received
 */
public class MessageMediaManager implements Runnable {
    Player player;
    
    /** Creates a new instance of MessageMediaManager */
    public MessageMediaManager(MIDlet midlet) throws SecurityException {
        InputStream is =
                midlet.getClass().getResourceAsStream("/res/newmessage.midi");
        try {
            player = Manager.createPlayer(is, "audio/midi");
            if (player==null) {
                Log.error("[MessageMediaManager] Player is null. Can't play " +
                        "sound. Probably security restrictions have been applied.");
                return;
            }
        } catch (IOException ex) {
            Log.error(this, "IOException in playSound: " + ex);
            ex.printStackTrace();
        } catch (MediaException ex) {
            Log.error(this, "MediaException in playSound: " + ex);
            ex.printStackTrace();
        } catch (IllegalArgumentException iae) {
            Log.error("[MessageMediaManager] Exception " +
                    "loading sound stream: " + iae);
            iae.printStackTrace();
        } catch (SecurityException se) {
            Log.error("[MessageMediaManager] The device carrier restricts " +
                    "Multi Media API usage to trusted applications only: " + se);
            UIController.ableToSound = false;
        } 
    }
    
    
    /**
     * Plays a sound (midi file from resources) for message received
     */
    public void playSound() {
        if (player!=null) {
            Log.info("play sound...");
            try {
                player.realize();
                Log.info("start sound...");
                player.start();
                Thread.sleep(3000);
            } catch (MediaException ex) {
                Log.error(this, "MediaException in playSound: " + ex);
                ex.printStackTrace();
            } catch (InterruptedException ex) {
                Log.error(this, "InterruptedException in playSound: " + ex);
                ex.printStackTrace();
            } catch (IllegalArgumentException iae) {
                Log.error("[MessageMediaManager] Exception " +
                        "loading sound stream: " + iae);
                iae.printStackTrace();
            } finally {
                if (player!=null) {
                    player.close();
                    player = null;
                }
            }
        }
    }
    
    /**
     * Run it in separate thread if needed
     */
    public void run() {
        playSound();
    }
}
