/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;
import com.funambol.mailclient.ui.controller.UIController;
import javax.microedition.lcdui.Font;

public class BasicGraphicalTheme implements GraphicalTheme {
    
    private int backgroundColor=UIController.display.getColor(UIController.display.COLOR_BACKGROUND);
    
    private int highlightedBackgroundColor=UIController.display.getColor(UIController.display.COLOR_HIGHLIGHTED_BACKGROUND);
    
    private int foregroundColor= UIController.display.getColor(UIController.display.COLOR_FOREGROUND);
    
    private int highlightedForegroundColor=UIController.display.getColor(UIController.display.COLOR_HIGHLIGHTED_FOREGROUND);
    
    private int borderColor= UIController.display.getColor(UIController.display.COLOR_BORDER);
    
    private int highlightedBorderColor = UIController.display.getColor(UIController.display.COLOR_HIGHLIGHTED_BORDER);
    
    
    
    /** Creates a new instance of Graphicaltheme with platform-coherent colors */
    public BasicGraphicalTheme() {
        
    }
    
    
    public int getBackgroundColor() {
        return verifyCorrectColor(backgroundColor, 0Xffffff);
    }
    
    public void setBackgroundColor(int backgroundColor) {
        this.backgroundColor = backgroundColor;
    }
    
    public int getHighlightedBackgroundColor() {
        return verifyCorrectColor(highlightedBackgroundColor, 0x000000);
    }
    
    public void setHighlightedBackgroundColor(int highlightedBackgroundColor) {
        this.highlightedBackgroundColor = highlightedBackgroundColor;
    }
    
    /*
     * using light gray for background scrollbar
     */
    public int getScrollbarBackgroundColor() {
        return 0xb2b2b2;
    }
    
    public int getForegroundColor() {
        return verifyCorrectColor(foregroundColor, 0X000000);
    }
    
    public void setForegroundColor(int foregroundColor) {
        this.foregroundColor = foregroundColor;
    }
    
    public int getHighlightedForegroundColor() {
        return verifyCorrectColor(highlightedForegroundColor, 0Xffffff);
    }
    
    public void setHighlightedForegroundColor(int higlightedForegroundColor) {
        this.highlightedForegroundColor = higlightedForegroundColor;
    }
    
    public int getBorderColor() {
        return borderColor;
    }
    
    public void setBorderColor(int borderColor) {
        this.borderColor = borderColor;
    }
    
    public int getHighlightedBorderColor() {
        return highlightedBorderColor;
    }
    
    public void setHighlightedBorderColor(int higlightedBorderColor) {
        this.highlightedBorderColor = higlightedBorderColor;
    }
    
    public Font getDefaultFont(){
        //return Font.getDefaultFont();
        return getSmallFont();
    }
    
    public Font getSmallFont() {
        return Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
    }
    
    public Font getBoldFont() {
        //return Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_MEDIUM);
        return getSmallBoldFont();
    }
    
    public int getTitleColor() {
        //return getBackgroundColor();
        // use white as color for title text
        return 0xFFFFFF;
    }
    
    public Font getTitleFont() {
     //   return getBoldFont();
     return Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_MEDIUM);
  
    }
    
    public int getTitleBackgroundColor() {
        //return getForegroundColor();
        // use blue from logo icon as color for title background
        return 0x0878a1;
    }
    
    public int getTitleBorderColor() {
        return getHighlightedBorderColor();
    }
    
    public Font getSmallBoldFont() {
        return Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_SMALL);
    }
    
    private int verifyCorrectColor(int color, int substituteColor) {
        if(backgroundColor == foregroundColor){
           return substituteColor;
       }
       else{
          return color;
       }
   }
}
